@@ -1,31 +1,8 @@
-/*
-  Copyright 2017 Larry Gritz et al. All Rights Reserved.
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// clang-format off
 
 #pragma once
 
@@ -64,7 +41,7 @@ OIIO_FORCEINLINE void clobber_all_memory();
 /// A call to clobber(p) fools the compiler into thinking that p (or *p, for
 /// the pointer version) might potentially have its memory altered. The
 /// implementation actually does nothing, but it's in another module, so the
-/// compiler won't know this and will be conservative about any assupmtions
+/// compiler won't know this and will be conservative about any assumptions
 /// of what's in p. This is helpful for benchmarking, to help erase any
 /// preconceptions the optimizer has about what might be in a variable.
 
@@ -74,6 +51,15 @@ OIIO_FORCEINLINE void clobber (const void* p) { clobber ((void*)p); }
 template<typename T>
 OIIO_FORCEINLINE T& clobber (T& p) { clobber(&p); return p; }
 
+// Multi-argument clobber, added in OIIO 2.2.2
+template<typename T, typename ...Ts>
+OIIO_FORCEINLINE void clobber (T& p, Ts&... ps)
+{
+    clobber(&p);
+    if (sizeof...(Ts) > 0)
+        clobber(ps...);
+}
+
 
 
 
@@ -89,7 +75,7 @@ OIIO_FORCEINLINE T& clobber (T& p) { clobber(&p); return p; }
 /// iterations based on their timing. For most use cases, it's fire and
 /// forget.
 ///
-/// Generally, the most and least expesive trials will be discarded (all
+/// Generally, the most and least expensive trials will be discarded (all
 /// sorts of things can happen to give you a few spurious results) and then
 /// the remainder of trials will be used to compute the average, standard
 /// deviation, range, and median value, in ns per iteration as well as
@@ -139,16 +125,17 @@ OIIO_FORCEINLINE T& clobber (T& p) { clobber(&p); return p; }
 
 class OIIO_API Benchmarker {
 public:
-    Benchmarker () {}
+    Benchmarker() {}
 
     // Calling Benchmarker like a function (operator()) executes the
     // benchmark. This process runs func(args...), several trials, each
     // trial with many iterations. The value returned is the best estimate
     // of the average time per iteration that it takes to run func.
-    template <typename FUNC, typename... ARGS>
-    double operator() (string_view name, FUNC func, ARGS&&... args) {
+    template<typename FUNC, typename... ARGS>
+    double operator()(string_view name, FUNC func, ARGS&&... args)
+    {
         m_name = name;
-        run (func, args...);
+        run(func, args...);
         if (verbose())
             std::cout << (*this) << std::endl;
         return avg();
@@ -156,20 +143,28 @@ public:
 
     // Return the average, sample standard deviation, median, and range
     // of per-iteration time.
-    double avg () const { return m_avg; }
-    double stddev () const { return m_stddev; }
-    double range () const { return m_range; }
-    double median () const { return m_median; }
+    double avg() const { return m_avg; }
+    double stddev() const { return m_stddev; }
+    double range() const { return m_range; }
+    double median() const { return m_median; }
 
     // Control the number of iterations per trial. The special value 0 means
     // to determine automatically a reasonable number of iterations. That is
     // also the default behavior.
-    Benchmarker& iterations (size_t val) { m_user_iterations = val; return *this; }
-    size_t iterations () const { return m_iterations; }
+    Benchmarker& iterations(size_t val)
+    {
+        m_user_iterations = val;
+        return *this;
+    }
+    size_t iterations() const { return m_iterations; }
 
     // Control the number of trials to perform.
-    Benchmarker& trials (size_t val) { m_trials = val; return *this; }
-    size_t trials () const { return m_trials; }
+    Benchmarker& trials(size_t val)
+    {
+        m_trials = val;
+        return *this;
+    }
+    size_t trials() const { return m_trials; }
 
     // Control the number of values of work that each iteration represents.
     // Usually you will leave this at the default of 1, but for some cases,
@@ -179,27 +174,43 @@ public:
     // 4 (or 8, etc.) times as many values. You can use the 'work' size to
     // make the calls report Mvals/s, showing more accurately than the SIMD
     // call is faster than the scalar call.
-    Benchmarker& work (size_t val) { m_work = val; return *this; }
-    size_t work () const { return m_work; }
+    Benchmarker& work(size_t val)
+    {
+        m_work = val;
+        return *this;
+    }
+    size_t work() const { return m_work; }
 
     // Control the exclusion of outliers. This number (default 1) of fastest
     // and slowest trials will be excluded from the statistics, to remove
     // the effects of spurious things happening on the system. Setting
     // outliers to 0 will compute statistics on all trials, without any
     // outlier exclusion.
-    Benchmarker& exclude_outliers (int e) { m_exclude_outliers = e; return *this; }
-    int exclude_outliers () const { return m_exclude_outliers; }
+    Benchmarker& exclude_outliers(int e)
+    {
+        m_exclude_outliers = e;
+        return *this;
+    }
+    int exclude_outliers() const { return m_exclude_outliers; }
 
     // Control the verbosity of the printing for each benchmark. The default
     // is 1, which prints basic statistics. Verbosity 0 is silent and leaves
     // it up to the app to retrieve results.
-    Benchmarker& verbose (int v) { m_verbose = v; return *this; }
-    int verbose () const { return m_verbose; }
+    Benchmarker& verbose(int v)
+    {
+        m_verbose = v;
+        return *this;
+    }
+    int verbose() const { return m_verbose; }
 
     // Control indentation in the printout -- this number of spaces will
     // be printed before the statistics.
-    Benchmarker& indent (int spaces) { m_indent = spaces; return *this; }
-    int indent () const { return m_indent; }
+    Benchmarker& indent(int spaces)
+    {
+        m_indent = spaces;
+        return *this;
+    }
+    int indent() const { return m_indent; }
 
     // Choices of unit to report results.
     enum class Unit : int { autounit, ns, us, ms, s };
@@ -208,63 +219,70 @@ public:
     // unit will be chosen for nice printing of each benchmark individually.
     // But the user may also wish to request specific units like ns or ux in
     // order to ensure that all benchmark resutls are using the same units.
-    Benchmarker& units (Unit s) { m_units = s; return *this; }
-    Unit units () const { return m_units; }
+    Benchmarker& units(Unit s)
+    {
+        m_units = s;
+        return *this;
+    }
+    Unit units() const { return m_units; }
 
-    const std::string& name () const { return m_name; }
+    const std::string& name() const { return m_name; }
 
 private:
-    size_t m_iterations = 0;
+    size_t m_iterations      = 0;
     size_t m_user_iterations = 0;
-    size_t m_trials = 10;
-    size_t m_work = 1;
+    size_t m_trials          = 10;
+    size_t m_work            = 1;
     std::string m_name;
     std::vector<double> m_times;  // times for each trial
-    double m_avg;     // average time per iteration
-    double m_stddev;  // standard deviation per iteration
-    double m_range;   // range per iteration
-    double m_median;  // median per-iteration time
+    double m_avg;                 // average time per iteration
+    double m_stddev;              // standard deviation per iteration
+    double m_range;               // range per iteration
+    double m_median;              // median per-iteration time
     int m_exclude_outliers = 1;
-    int m_verbose = 1;
-    int m_indent = 0;
-    Unit m_units = Unit::autounit;
+    int m_verbose          = 1;
+    int m_indent           = 0;
+    Unit m_units           = Unit::autounit;
 
-    template <typename FUNC, typename... ARGS>
-    double run (FUNC func, ARGS&&... args) {
+    template<typename FUNC, typename... ARGS>
+    double run(FUNC func, ARGS&&... args)
+    {
         if (m_user_iterations)
             m_iterations = m_user_iterations;
         else
             m_iterations = determine_iterations(func, args...);
-        m_times.resize (m_trials);
+        m_times.resize(m_trials);
 
         double overhead = iteration_overhead() * iterations();
         for (auto& t : m_times)
-            t = std::max (0.0, do_trial (m_iterations, func, args...) - overhead);
-        compute_stats ();
+            t = std::max(0.0, do_trial(m_iterations, func, args...) - overhead);
+        compute_stats();
         return avg();
     }
 
-    template <typename FUNC, typename... ARGS>
-    size_t determine_iterations (FUNC func, ARGS&&... args) {
+    template<typename FUNC, typename... ARGS>
+    size_t determine_iterations(FUNC func, ARGS&&... args)
+    {
         // We're shooting for a trial around 1/100s
         const double target_time = 0.01;
         size_t i = 1;
         while (1) {
             double t = do_trial (i, func, args...);
             // std::cout << "Trying " << i << " iters = " << t << "\n";
-            if (t > target_time*1.5 && i > 2)
-                return i/2;
-            if (t > target_time*0.75 || i > (size_t(1)<<30))
+            if (t > target_time * 1.5 && i > 2)
+                return i / 2;
+            if (t > target_time * 0.75 || i > (size_t(1) << 30))
                 return i;
-            if (t < target_time/16)
+            if (t < target_time / 16)
                 i *= 8;
             else
                 i *= 2;
         }
     }
 
-    template <typename FUNC, typename... ARGS>
-    double do_trial (size_t iterations, FUNC func, ARGS&&... args) {
+    template<typename FUNC, typename... ARGS>
+    double do_trial(size_t iterations, FUNC func, ARGS&&... args)
+    {
         Timer timer;
         while (iterations--) {
             clobber_all_memory();
@@ -273,16 +291,16 @@ private:
         return timer();
     }
 
-    void compute_stats () { compute_stats (m_times, m_iterations); }
-    void compute_stats (std::vector<double>& times, size_t iterations);
+    void compute_stats() { compute_stats(m_times, m_iterations); }
+    void compute_stats(std::vector<double>& times, size_t iterations);
     double iteration_overhead();
 
-    friend OIIO_API std::ostream& operator<< (std::ostream& out, const Benchmarker &bench);
+    friend OIIO_API std::ostream& operator<<(std::ostream& out,
+                                             const Benchmarker& bench);
 };
 
 
 
-
 /// Helper template that runs a function (or functor) n times, using a
 /// Timer to benchmark the results, and returning the fastest trial.  If
 /// 'range' is non-NULL, the range (max-min) of the various time trials
@@ -292,15 +310,15 @@ private:
 /// Benchmarker class is a better solution.
 template<typename FUNC>
 double
-time_trial (FUNC func, int ntrials=1, int nrepeats = 1, double *range=NULL)
+time_trial(FUNC func, int ntrials = 1, int nrepeats = 1, double* range = NULL)
 {
     double mintime = 1.0e30, maxtime = 0.0;
     while (ntrials-- > 0) {
         Timer timer;
         for (int i = 0; i < nrepeats; ++i) {
-            // Be sure that the repeated calls to func aren't optimzed away:
+            // Be sure that the repeated calls to func aren't optimized away:
             clobber_all_memory();
-            func ();
+            func();
         }
         double t = timer();
         if (t < mintime)
@@ -309,14 +327,16 @@ time_trial (FUNC func, int ntrials=1, int nrepeats = 1, double *range=NULL)
             maxtime = t;
     }
     if (range)
-        *range = maxtime-mintime;
+        *range = maxtime - mintime;
     return mintime;
 }
 
 /// Version without repeats.
 template<typename FUNC>
-double time_trial (FUNC func, int ntrials, double *range) {
-    return time_trial (func, ntrials, 1, range);
+double
+time_trial(FUNC func, int ntrials, double* range)
+{
+    return time_trial(func, ntrials, 1, range);
 }
 
 
@@ -336,7 +356,7 @@ double time_trial (FUNC func, int ntrials, double *range) {
 //     ntrials : The number of runs for each thread count (more will take
 //                      longer, but be more accurate timing). The best case
 //                      run is the one that will be reported.
-//     threadcounts[] : An array_view<int> giving the set of thread counts
+//     threadcounts[] : An span<int> giving the set of thread counts
 //                      to try.
 // Return value:
 //     A vector<double> containing the best time (of the trials) for each
@@ -348,7 +368,7 @@ timed_thread_wedge (function_view<void(int)> task,
                     std::ostream *out,
                     int maxthreads,
                     int total_iterations, int ntrials,
-                    array_view<const int> threadcounts = {1,2,4,8,12,16,24,32,48,64,128});
+                    cspan<int> threadcounts = {1,2,4,8,12,16,24,32,48,64,128});
 
 // Simplified timed_thread_wedge without pre- and post-tasks, using
 // std::out for output, with a default set of thread counts, and not needing
@@ -356,7 +376,7 @@ timed_thread_wedge (function_view<void(int)> task,
 OIIO_API void
 timed_thread_wedge (function_view<void(int)> task,
                     int maxthreads, int total_iterations, int ntrials,
-                    array_view<const int> threadcounts = {1,2,4,8,12,16,24,32,48,64,128});
+                    cspan<int> threadcounts = {1,2,4,8,12,16,24,32,48,64,128});
 
 
 
@@ -407,7 +427,7 @@ OIIO_FORCEINLINE T const & DoNotOptimize (T const &val) {
 // If __optnone__ attribute is available: make a null function with no
 // optimization, that's all we need.
 template <class T>
-OIIO_FORCEINLINE T const & __attribute__((__optnone__))
+inline T const & __attribute__((__optnone__))
 DoNotOptimize (T const &val) {
     return val;
 }
@@ -447,6 +467,4 @@ OIIO_FORCEINLINE void clobber_all_memory() { }
 
 
 
-
 OIIO_NAMESPACE_END
-
